@@ -9,11 +9,11 @@ import textwrap
 import types
 
 import attr
-import importlib_metadata
 import py
 import six
 
 import pytest
+from _pytest.compat import importlib_metadata
 from _pytest.main import EXIT_NOTESTSCOLLECTED
 from _pytest.main import EXIT_USAGEERROR
 from _pytest.warnings import SHOW_PYTEST_WARNINGS_ARG
@@ -223,7 +223,7 @@ class TestGeneralUsage(object):
                 "conftest.py:2: in foo",
                 "    import qwerty",
                 "E   {}: No module named {q}qwerty{q}".format(
-                    exc_name, q="'" if six.PY3 else ""
+                    exc_name, q="" if six.PY2 else "'"
                 ),
             ]
         )
