@@ -54,6 +54,7 @@ def test_root_logger_affected(testdir):
         """
         import logging
         logger = logging.getLogger()
+
         def test_foo():
             logger.info('info text ' + 'going to logger')
             logger.warning('warning text ' + 'going to logger')
@@ -66,15 +67,14 @@ def test_root_logger_affected(testdir):
     result = testdir.runpytest("--log-level=ERROR", "--log-file=pytest.log")
     assert result.ret == 1
 
-    # the capture log calls in the stdout section only contain the
-    # logger.error msg, because --log-level=ERROR
+    # The capture log calls in the stdout section only contain the
+    # logger.error msg, because of --log-level=ERROR.
     result.stdout.fnmatch_lines(["*error text going to logger*"])
-    with pytest.raises(pytest.fail.Exception):
-        result.stdout.fnmatch_lines(["*warning text going to logger*"])
-    with pytest.raises(pytest.fail.Exception):
-        result.stdout.fnmatch_lines(["*info text going to logger*"])
+    stdout = result.stdout.str()
+    assert "warning text going to logger" not in stdout
+    assert "info text going to logger" not in stdout
 
-    # the log file should contain the warning and the error log messages and
+    # The log file should contain the warning and the error log messages and
     # not the info one, because the default level of the root logger is
     # WARNING.
     assert os.path.isfile(log_file)
@@ -248,7 +248,7 @@ def test_log_cli_enabled_disabled(testdir, enabled):
             [
                 "test_log_cli_enabled_disabled.py::test_log_cli ",
                 "*-- live log call --*",
-                "test_log_cli_enabled_disabled.py* CRITICAL critical message logged by test",
+                "CRITICAL *test_log_cli_enabled_disabled.py* critical message logged by test",
                 "PASSED*",
             ]
         )
@@ -282,7 +282,7 @@ def test_log_cli_default_level(testdir):
     result.stdout.fnmatch_lines(
         [
             "test_log_cli_default_level.py::test_log_cli ",
-            "test_log_cli_default_level.py*WARNING message will be shown*",
+            "WARNING*test_log_cli_default_level.py* message will be shown*",
         ]
     )
     assert "INFO message won't be shown" not in result.stdout.str()
@@ -523,7 +523,7 @@ def test_sections_single_new_line_after_test_outcome(testdir, request):
     )
     assert (
         re.search(
-            r"(.+)live log teardown(.+)\n(.+)WARNING(.+)\n(.+)WARNING(.+)",
+            r"(.+)live log teardown(.+)\nWARNING(.+)\nWARNING(.+)",
             result.stdout.str(),
             re.MULTILINE,
         )
@@ -531,7 +531,7 @@ def test_sections_single_new_line_after_test_outcome(testdir, request):
     )
     assert (
         re.search(
-            r"(.+)live log finish(.+)\n(.+)WARNING(.+)\n(.+)WARNING(.+)",
+            r"(.+)live log finish(.+)\nWARNING(.+)\nWARNING(.+)",
             result.stdout.str(),
             re.MULTILINE,
         )
@@ -565,7 +565,7 @@ def test_log_cli_level(testdir):
     # fnmatch_lines does an assertion internally
     result.stdout.fnmatch_lines(
         [
-            "test_log_cli_level.py*This log message will be shown",
+            "*test_log_cli_level.py*This log message will be shown",
             "PASSED",  # 'PASSED' on its own line because the log message prints a new line
         ]
     )
@@ -579,7 +579,7 @@ def test_log_cli_level(testdir):
     # fnmatch_lines does an assertion internally
     result.stdout.fnmatch_lines(
         [
-            "test_log_cli_level.py* This log message will be shown",
+            "*test_log_cli_level.py* This log message will be shown",
             "PASSED",  # 'PASSED' on its own line because the log message prints a new line
         ]
     )
@@ -615,7 +615,7 @@ def test_log_cli_ini_level(testdir):
     # fnmatch_lines does an assertion internally
     result.stdout.fnmatch_lines(
         [
-            "test_log_cli_ini_level.py* This log message will be shown",
+            "*test_log_cli_ini_level.py* This log message will be shown",
             "PASSED",  # 'PASSED' on its own line because the log message prints a new line
         ]
     )
@@ -635,7 +635,6 @@ def test_log_cli_auto_enable(testdir, request, cli_args):
     """
     testdir.makepyfile(
         """
-        import pytest
         import logging
 
         def test_log_1():
@@ -653,6 +652,7 @@ def test_log_cli_auto_enable(testdir, request, cli_args):
     )
 
     result = testdir.runpytest(cli_args)
+    stdout = result.stdout.str()
     if cli_args == "--log-cli-level=WARNING":
         result.stdout.fnmatch_lines(
             [
@@ -663,13 +663,13 @@ def test_log_cli_auto_enable(testdir, request, cli_args):
                 "=* 1 passed in *=",
             ]
         )
-        assert "INFO" not in result.stdout.str()
+        assert "INFO" not in stdout
     else:
         result.stdout.fnmatch_lines(
             ["*test_log_cli_auto_enable*100%*", "=* 1 passed in *="]
         )
-        assert "INFO" not in result.stdout.str()
-        assert "WARNING" not in result.stdout.str()
+        assert "INFO" not in stdout
+        assert "WARNING" not in stdout
 
 
 def test_log_file_cli(testdir):
@@ -747,7 +747,7 @@ def test_log_level_not_changed_by_default(testdir):
     """
     )
     result = testdir.runpytest("-s")
-    result.stdout.fnmatch_lines("* 1 passed in *")
+    result.stdout.fnmatch_lines(["* 1 passed in *"])
 
 
 def test_log_file_ini(testdir):
@@ -921,15 +921,46 @@ def test_collection_live_logging(testdir):
 
     result = testdir.runpytest("--log-cli-level=INFO")
     result.stdout.fnmatch_lines(
-        [
-            "collecting*",
-            "*--- live log collection ---*",
-            "*Normal message*",
-            "collected 0 items",
-        ]
+        ["*--- live log collection ---*", "*Normal message*", "collected 0 items"]
     )
 
 
+@pytest.mark.parametrize("verbose", ["", "-q", "-qq"])
+def test_collection_collect_only_live_logging(testdir, verbose):
+    testdir.makepyfile(
+        """
+        def test_simple():
+            pass
+    """
+    )
+
+    result = testdir.runpytest("--collect-only", "--log-cli-level=INFO", verbose)
+
+    expected_lines = []
+
+    if not verbose:
+        expected_lines.extend(
+            [
+                "*collected 1 item*",
+                "*<Module test_collection_collect_only_live_logging.py>*",
+                "*no tests ran*",
+            ]
+        )
+    elif verbose == "-q":
+        assert "collected 1 item*" not in result.stdout.str()
+        expected_lines.extend(
+            [
+                "*test_collection_collect_only_live_logging.py::test_simple*",
+                "no tests ran in * seconds",
+            ]
+        )
+    elif verbose == "-qq":
+        assert "collected 1 item*" not in result.stdout.str()
+        expected_lines.extend(["*test_collection_collect_only_live_logging.py: 1*"])
+
+    result.stdout.fnmatch_lines(expected_lines)
+
+
 def test_collection_logging_to_file(testdir):
     log_file = testdir.tmpdir.join("pytest.log").strpath
 
@@ -1084,3 +1115,48 @@ def test_log_set_path(testdir):
     with open(os.path.join(report_dir_base, "test_second"), "r") as rfh:
         content = rfh.read()
         assert "message from test 2" in content
+
+
+def test_colored_captured_log(testdir):
+    """
+    Test that the level names of captured log messages of a failing test are
+    colored.
+    """
+    testdir.makepyfile(
+        """
+        import logging
+
+        logger = logging.getLogger(__name__)
+
+        def test_foo():
+            logger.info('text going to logger from call')
+            assert False
+        """
+    )
+    result = testdir.runpytest("--log-level=INFO", "--color=yes")
+    assert result.ret == 1
+    result.stdout.fnmatch_lines(
+        [
+            "*-- Captured log call --*",
+            "\x1b[32mINFO    \x1b[0m*text going to logger from call",
+        ]
+    )
+
+
+def test_colored_ansi_esc_caplogtext(testdir):
+    """
+    Make sure that caplog.text does not contain ANSI escape sequences.
+    """
+    testdir.makepyfile(
+        """
+        import logging
+
+        logger = logging.getLogger(__name__)
+
+        def test_foo(caplog):
+            logger.info('text going to logger from call')
+            assert '\x1b' not in caplog.text
+        """
+    )
+    result = testdir.runpytest("--log-level=INFO", "--color=yes")
+    assert result.ret == 0
