@@ -34,8 +34,6 @@ class TestModule(object):
         )
 
     def test_import_prepend_append(self, testdir, monkeypatch):
-        syspath = list(sys.path)
-        monkeypatch.setattr(sys, "path", syspath)
         root1 = testdir.mkdir("root1")
         root2 = testdir.mkdir("root2")
         root1.ensure("x456.py")
@@ -148,7 +146,24 @@ class TestClass(object):
         result = testdir.runpytest("-rw")
         result.stdout.fnmatch_lines(
             [
-                "*cannot collect test class 'TestClass1' because it has a __init__ constructor"
+                "*cannot collect test class 'TestClass1' because it has "
+                "a __init__ constructor (from: test_class_with_init_warning.py)"
+            ]
+        )
+
+    def test_class_with_new_warning(self, testdir):
+        testdir.makepyfile(
+            """
+            class TestClass1(object):
+                def __new__(self):
+                    pass
+        """
+        )
+        result = testdir.runpytest("-rw")
+        result.stdout.fnmatch_lines(
+            [
+                "*cannot collect test class 'TestClass1' because it has "
+                "a __new__ constructor (from: test_class_with_new_warning.py)"
             ]
         )
 
@@ -560,7 +575,7 @@ class TestFunction(object):
         """
         )
         result = testdir.runpytest()
-        result.stdout.fnmatch_lines("* 2 passed, 1 skipped in *")
+        result.stdout.fnmatch_lines(["* 2 passed, 1 skipped in *"])
 
     def test_parametrize_skip(self, testdir):
         testdir.makepyfile(
@@ -575,7 +590,7 @@ class TestFunction(object):
         """
         )
         result = testdir.runpytest()
-        result.stdout.fnmatch_lines("* 2 passed, 1 skipped in *")
+        result.stdout.fnmatch_lines(["* 2 passed, 1 skipped in *"])
 
     def test_parametrize_skipif_no_skip(self, testdir):
         testdir.makepyfile(
@@ -590,7 +605,7 @@ class TestFunction(object):
         """
         )
         result = testdir.runpytest()
-        result.stdout.fnmatch_lines("* 1 failed, 2 passed in *")
+        result.stdout.fnmatch_lines(["* 1 failed, 2 passed in *"])
 
     def test_parametrize_xfail(self, testdir):
         testdir.makepyfile(
@@ -605,7 +620,7 @@ class TestFunction(object):
         """
         )
         result = testdir.runpytest()
-        result.stdout.fnmatch_lines("* 2 passed, 1 xfailed in *")
+        result.stdout.fnmatch_lines(["* 2 passed, 1 xfailed in *"])
 
     def test_parametrize_passed(self, testdir):
         testdir.makepyfile(
@@ -620,7 +635,7 @@ class TestFunction(object):
         """
         )
         result = testdir.runpytest()
-        result.stdout.fnmatch_lines("* 2 passed, 1 xpassed in *")
+        result.stdout.fnmatch_lines(["* 2 passed, 1 xpassed in *"])
 
     def test_parametrize_xfail_passed(self, testdir):
         testdir.makepyfile(
@@ -635,7 +650,7 @@ class TestFunction(object):
         """
         )
         result = testdir.runpytest()
-        result.stdout.fnmatch_lines("* 3 passed in *")
+        result.stdout.fnmatch_lines(["* 3 passed in *"])
 
     def test_function_original_name(self, testdir):
         items = testdir.getitems(
@@ -833,7 +848,7 @@ class TestConftestCustomization(object):
         )
         # Use runpytest_subprocess, since we're futzing with sys.meta_path.
         result = testdir.runpytest_subprocess()
-        result.stdout.fnmatch_lines("*1 passed*")
+        result.stdout.fnmatch_lines(["*1 passed*"])
 
 
 def test_setup_only_available_in_subdir(testdir):
@@ -1246,7 +1261,7 @@ def test_syntax_error_with_non_ascii_chars(testdir):
     """
     testdir.makepyfile(
         u"""
-    # -*- coding: UTF-8 -*-
+    # -*- coding: utf-8 -*-
 
     ☃
     """
@@ -1298,14 +1313,14 @@ def test_keep_duplicates(testdir):
 def test_package_collection_infinite_recursion(testdir):
     testdir.copy_example("collect/package_infinite_recursion")
     result = testdir.runpytest()
-    result.stdout.fnmatch_lines("*1 passed*")
+    result.stdout.fnmatch_lines(["*1 passed*"])
 
 
 def test_package_collection_init_given_as_argument(testdir):
     """Regression test for #3749"""
     p = testdir.copy_example("collect/package_init_given_as_arg")
     result = testdir.runpytest(p / "pkg" / "__init__.py")
-    result.stdout.fnmatch_lines("*1 passed*")
+    result.stdout.fnmatch_lines(["*1 passed*"])
 
 
 def test_package_with_modules(testdir):
