@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import pytest
 from _pytest import python
 from _pytest import runner
@@ -283,7 +284,7 @@ class TestReRunTests(object):
                 global count, req
                 assert request != req
                 req = request
-                print ("fix count %s" % count)
+                print("fix count %s" % count)
                 count += 1
             def test_fix(fix):
                 pass
@@ -393,8 +394,9 @@ class TestNoselikeTestAttribute(object):
         assert not call.items
 
 
-@pytest.mark.issue351
 class TestParameterize(object):
+    """#351"""
+
     def test_idfn_marker(self, testdir):
         testdir.makepyfile(
             """
