@@ -14,6 +14,7 @@ import pytest
 from _pytest import outcomes
 from _pytest.assertion import truncate
 from _pytest.assertion import util
+from _pytest.compat import ATTRS_EQ_FIELD
 
 PY3 = sys.version_info >= (3, 0)
 
@@ -179,7 +180,8 @@ class TestImportHookInstallation(object):
                 return check
             """,
             "mainwrapper.py": """\
-            import pytest, importlib_metadata
+            import pytest
+            from _pytest.compat import importlib_metadata
 
             class DummyEntryPoint(object):
                 name = 'spam'
@@ -687,7 +689,7 @@ class TestAssert_reprcompare_attrsclass(object):
         @attr.s
         class SimpleDataObject(object):
             field_a = attr.ib()
-            field_b = attr.ib(cmp=False)
+            field_b = attr.ib(**{ATTRS_EQ_FIELD: False})
 
         left = SimpleDataObject(1, "b")
         right = SimpleDataObject(1, "b")
