@@ -106,7 +106,7 @@ def test_capturing_unicode(testdir, method):
         obj = "u'\u00f6y'"
     testdir.makepyfile(
         """
-        # coding=utf8
+        # -*- coding: utf-8 -*-
         # taken from issue 227 from nosetests
         def test_unicode():
             import sys
@@ -134,12 +134,22 @@ def test_capturing_bytes_in_utf8_encoding(testdir, method):
 def test_collect_capturing(testdir):
     p = testdir.makepyfile(
         """
+        import sys
+
         print("collect %s failure" % 13)
+        sys.stderr.write("collect %s_stderr failure" % 13)
         import xyz42123
     """
     )
     result = testdir.runpytest(p)
-    result.stdout.fnmatch_lines(["*Captured stdout*", "*collect 13 failure*"])
+    result.stdout.fnmatch_lines(
+        [
+            "*Captured stdout*",
+            "collect 13 failure",
+            "*Captured stderr*",
+            "collect 13_stderr failure",
+        ]
+    )
 
 
 class TestPerTestCapturing(object):
@@ -566,7 +576,7 @@ class TestCaptureFixture(object):
         result.stdout.fnmatch_lines(
             [
                 "*test_hello*",
-                "*capsysbinary is only supported on python 3*",
+                "*capsysbinary is only supported on Python 3*",
                 "*1 error in*",
             ]
         )
@@ -595,8 +605,8 @@ class TestCaptureFixture(object):
         result.stdout.fnmatch_lines(["*KeyboardInterrupt*"])
         assert result.ret == 2
 
-    @pytest.mark.issue14
     def test_capture_and_logging(self, testdir):
+        """#14"""
         p = testdir.makepyfile(
             """\
             import logging
@@ -673,7 +683,7 @@ class TestCaptureFixture(object):
             )
         )
         result = testdir.runpytest_subprocess()
-        result.stdout.fnmatch_lines("*1 passed*")
+        result.stdout.fnmatch_lines(["*1 passed*"])
         assert "stdout contents begin" not in result.stdout.str()
         assert "stderr contents begin" not in result.stdout.str()
 
@@ -809,15 +819,15 @@ def test_error_during_readouterr(testdir):
     testdir.makepyfile(
         pytest_xyz="""
         from _pytest.capture import FDCapture
+
         def bad_snap(self):
             raise Exception('boom')
+
         assert FDCapture.snap
         FDCapture.snap = bad_snap
     """
     )
-    result = testdir.runpytest_subprocess(
-        "-p", "pytest_xyz", "--version", syspathinsert=True
-    )
+    result = testdir.runpytest_subprocess("-p", "pytest_xyz", "--version")
     result.stderr.fnmatch_lines(
         ["*in bad_snap", "    raise Exception('boom')", "Exception: boom"]
     )
@@ -1041,6 +1051,9 @@ class TestFDCapture(object):
             cap.done()
             pytest.raises(AttributeError, cap.suspend)
 
+    def test_capfd_sys_stdout_mode(self, capfd):
+        assert "b" not in sys.stdout.mode
+
 
 @contextlib.contextmanager
 def saved_fd(fd):
@@ -1231,20 +1244,26 @@ class TestStdCaptureFDinvalidFD(object):
             """
             import os
             from _pytest import capture
+
             def StdCaptureFD(out=True, err=True, in_=True):
-                return capture.MultiCapture(out, err, in_,
-                                              Capture=capture.FDCapture)
+                return capture.MultiCapture(out, err, in_, Capture=capture.FDCapture)
+
             def test_stdout():
                 os.close(1)
                 cap = StdCaptureFD(out=True, err=False, in_=False)
+                assert repr(cap.out) == "<FDCapture 1 oldfd=None _state=None>"
                 cap.stop_capturing()
+
             def test_stderr():
                 os.close(2)
                 cap = StdCaptureFD(out=False, err=True, in_=False)
+                assert repr(cap.err) == "<FDCapture 2 oldfd=None _state=None>"
                 cap.stop_capturing()
+
             def test_stdin():
                 os.close(0)
                 cap = StdCaptureFD(out=False, err=False, in_=True)
+                assert repr(cap.in_) == "<FDCapture 0 oldfd=None _state=None>"
                 cap.stop_capturing()
         """
         )
