@@ -3,7 +3,7 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-import importlib_metadata
+from _pytest.compat import importlib_metadata
 
 
 def test_pytest_entry_points_are_identical():
