@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 import py
 
 import pytest
@@ -10,11 +11,11 @@ from _pytest import nodes
         ("", "", True),
         ("", "foo", True),
         ("", "foo/bar", True),
-        ("", "foo/bar::TestBaz::()", True),
+        ("", "foo/bar::TestBaz", True),
         ("foo", "food", False),
-        ("foo/bar::TestBaz::()", "foo/bar", False),
-        ("foo/bar::TestBaz::()", "foo/bar::TestBop::()", False),
-        ("foo/bar", "foo/bar::TestBop::()", True),
+        ("foo/bar::TestBaz", "foo/bar", False),
+        ("foo/bar::TestBaz", "foo/bar::TestBop", False),
+        ("foo/bar", "foo/bar::TestBop", True),
     ),
 )
 def test_ischildnode(baseid, nodeid, expected):
