@@ -74,7 +74,7 @@ class TestPasteCapture(object):
         """
         )
         result = testdir.runpytest("--pastebin=all")
-        if sys.version_info[0] == 3:
+        if sys.version_info[0] >= 3:
             expected_msg = "*assert '☺' == 1*"
         else:
             expected_msg = "*assert '\\xe2\\x98\\xba' == 1*"
@@ -126,7 +126,7 @@ class TestPaste(object):
         assert len(mocked_urlopen) == 1
         url, data = mocked_urlopen[0]
         assert type(data) is bytes
-        lexer = "python3" if sys.version_info[0] == 3 else "python"
+        lexer = "text"
         assert url == "https://bpaste.net"
         assert "lexer=%s" % lexer in data.decode()
         assert "code=full-paste-contents" in data.decode()
