@@ -1,4 +1,4 @@
-# encoding: UTF-8
+# -*- coding: utf-8 -*-
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
@@ -9,6 +9,7 @@ import types
 
 import pytest
 from _pytest.config import PytestPluginManager
+from _pytest.config.exceptions import UsageError
 from _pytest.main import EXIT_NOTESTSCOLLECTED
 from _pytest.main import Session
 
@@ -154,7 +155,7 @@ def test_importplugin_error_message(testdir, pytestpm):
     testdir.syspathinsert(testdir.tmpdir)
     testdir.makepyfile(
         qwe="""
-        # encoding: UTF-8
+        # -*- coding: utf-8 -*-
         def test_traceback():
             raise ImportError(u'Not possible to import: ☺')
         test_traceback()
@@ -314,6 +315,9 @@ class TestPytestPluginManagerBootstrapming(object):
         # Handles -p without following arg (when used without argparse).
         pytestpm.consider_preparse(["-p"])
 
+        with pytest.raises(UsageError, match="^plugin main cannot be disabled$"):
+            pytestpm.consider_preparse(["-p", "no:main"])
+
     def test_plugin_prevent_register(self, pytestpm):
         pytestpm.consider_preparse(["xyz", "-p", "no:abc"])
         l1 = pytestpm.get_plugins()
