@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
@@ -68,9 +69,7 @@ class SessionTests(object):
         passed, skipped, failed = reprec.listoutcomes()
         assert len(failed) == 1
         out = failed[0].longrepr.reprcrash.message
-        if not out.find("DID NOT RAISE") != -1:
-            print(out)
-            pytest.fail("incorrect raises() output")
+        assert "DID NOT RAISE" in out
 
     def test_syntax_error_module(self, testdir):
         reprec = testdir.inline_runsource("this is really not python")
@@ -116,7 +115,8 @@ class SessionTests(object):
             class TestBrokenClass(object):
                 def test_explicit_bad_repr(self):
                     t = BrokenRepr1()
-                    pytest.raises(Exception, 'repr(t)')
+                    with pytest.raises(Exception, match="I'm a broken repr"):
+                        repr(t)
 
                 def test_implicit_bad_repr1(self):
                     t = BrokenRepr1()
@@ -126,14 +126,38 @@ class SessionTests(object):
         )
         reprec = testdir.inline_run(p)
         passed, skipped, failed = reprec.listoutcomes()
-        assert len(failed) == 1
+        assert (len(passed), len(skipped), len(failed)) == (1, 0, 1)
         out = failed[0].longrepr.reprcrash.message
         assert (
             out.find(
                 """[Exception("Ha Ha fooled you, I'm a broken repr().") raised in repr()]"""
             )
             != -1
-        )  # '
+        )
+
+    def test_broken_repr_with_showlocals_verbose(self, testdir):
+        p = testdir.makepyfile(
+            """
+            class ObjWithErrorInRepr:
+                def __repr__(self):
+                    raise NotImplementedError
+
+            def test_repr_error():
+                x = ObjWithErrorInRepr()
+                assert x == "value"
+        """
+        )
+        reprec = testdir.inline_run("--showlocals", "-vv", p)
+        passed, skipped, failed = reprec.listoutcomes()
+        assert (len(passed), len(skipped), len(failed)) == (0, 0, 1)
+        entries = failed[0].longrepr.reprtraceback.reprentries
+        assert len(entries) == 1
+        repr_locals = entries[0].reprlocals
+        assert repr_locals.lines
+        assert len(repr_locals.lines) == 1
+        assert repr_locals.lines[0].startswith(
+            'x          = <[NotImplementedError("") raised in repr()] ObjWithErrorInRepr'
+        )
 
     def test_skip_file_by_conftest(self, testdir):
         testdir.makepyfile(
@@ -148,7 +172,7 @@ class SessionTests(object):
         )
         try:
             reprec = testdir.inline_run(testdir.tmpdir)
-        except pytest.skip.Exception:
+        except pytest.skip.Exception:  # pragma: no cover
             pytest.fail("wrong skipped caught")
         reports = reprec.getreports("pytest_collectreport")
         assert len(reports) == 1
@@ -332,7 +356,7 @@ def test_rootdir_option_arg(testdir, monkeypatch, path):
     result = testdir.runpytest("--rootdir={}".format(path))
     result.stdout.fnmatch_lines(
         [
-            "*rootdir: {}/root, inifile:*".format(testdir.tmpdir),
+            "*rootdir: {}/root".format(testdir.tmpdir),
             "root/test_rootdir_option_arg.py *",
             "*1 passed*",
         ]
